@@ -1,3 +1,5 @@
+__version__ = '0.5.0'
+
 from .code import Code
 from .codepack import CodePack
 from .arg import Arg
@@ -9,19 +11,12 @@ from .utils.config.alias import Alias
 from .utils.looper import Looper
 from .utils.common import Common
 
-from .plugins.worker import Worker
-from .plugins.supervisor import Supervisor
-from .plugins.docker_manager import DockerManager
-from .plugins.interpreter_manager import InterpreterManager
 from .plugins.dependency_bag import DependencyBag
 from .plugins.dependency_monitor import DependencyMonitor
 from .plugins.storage_service import StorageService
 from .plugins.snapshot_service import SnapshotService
 from .plugins.delivery_service import DeliveryService
 from .plugins.callback_service import CallbackService
-from .plugins.scheduler import Scheduler
-from .plugins.jobstore import JobStore
-from .plugins.storable_job import StorableJob
 from .plugins.dependency import Dependency
 from .plugins.delivery import Delivery
 from .plugins.callback import Callback
@@ -29,3 +24,13 @@ from .plugins.snapshots.snapshot import Snapshot
 from .plugins.snapshots.code_snapshot import CodeSnapshot
 from .plugins.snapshots.codepack_snapshot import CodePackSnapshot
 from .plugins.state import State
+try:
+    from .plugins.worker import Worker
+    from .plugins.supervisor import Supervisor
+    from .plugins.docker_manager import DockerManager
+    from .plugins.interpreter_manager import InterpreterManager
+    from .plugins.scheduler import Scheduler
+    from .plugins.jobstore import JobStore
+    from .plugins.storable_job import StorableJob
+except Exception:
+    pass
